/*
 *	Client routines used to negotiate PKE facility.
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifdef PKE_ENCRYPTION
#ifndef lint
static char RCSid[] = "$Id: pkeclnt.c,v 1.15 1997/02/08 07:14:01 sjg Exp $";
#endif

#include <sys/types.h>
#include <arpa/telnet.h>

#include <ctype.h>

#include "ring.h"

#include "defines.h"
#include "externs.h"
#include "types.h"
#include "general.h"

#include "pkepub.h"

static int sent_key, asked_key;

int
pke_connect(connected)
	int connected;
{
	sent_key = asked_key = 0;	/* can't be on yet... */
	pke_off();
	if (connected) {
#if defined(AUTHENTICATION)
		if (autologin) {
			/* want this one first */
			set_my_state_dont(TELOPT_AUTHENTICATION);
			send_will(TELOPT_AUTHENTICATION, 1);
		}
#endif
		if (my_want_state_is_will(TELOPT_PKE)) {
			set_my_state_dont(TELOPT_PKE); /* make sure we send it */
			send_will(TELOPT_PKE, 1);
		}
	}		
	return 0;
}

static void
send_key(st)
	int st;
{
	/* the 2 below days we support read/write keys */
	static unsigned char sb[] = {
		IAC, SB, TELOPT_PKE, TELQUAL_SEND, 2, IAC, SE
	};
	if (st & PKE_KEYSET)
		sent_key = 1;
	
#if defined(AUTHENTICATION)
	/*
	 * if we have agreed to do AUTHENTICATION
	 * then we do not want to exchange pub keys
	 */
	if (autologin &&
	    auth_status_any() &&
	    (his_state_is_will(TELOPT_AUTHENTICATION) ||
	     his_state_is_do(TELOPT_AUTHENTICATION))) {
			asked_key = 1;
	}
#endif
	if (asked_key == 0) {
		writenet(sb, sizeof(sb));
		asked_key = 1;
		printsub('>', &sb[2], sizeof(sb) - 2);
	} else if (sent_key == 1) {
		sent_key++;
		send_will(TELOPT_PKE, 0);
	} 
}

/*
 * We received DO... the first time we get it we send a
 * request for a key.
 * We should receive it again after we have sent the private key,
 * which is the server confirming that we shoud turn it on...
 */

int
pke_will()
{
#ifdef AUTH_HPW
	extern char *MyId;
#endif
	extern char do_dont_resp[256];
	int doit = 1;
	int st;
	char *p = NULL;

	st = pke_status(NULL);

	if ((st & PKE_OK) == 0) {
		doit = 0;
	} else if (sent_key) {
		/*
		 * It looks silly, but we need to try and synch the point
		 * at which we turn encryption on...
		 */
		pke_on();
		do_dont_resp[TELOPT_PKE] = 1;	/* need to keep quite */
		doit = 0;
	}
#ifdef AUTH_HPW
	if (doit == 0) {			/* last WILL */
		st = pke_status(&p);
		printf("%s: %s", MyId, &p[2]);
	} else if (sent_key == 0) 
#endif
		send_key(st);
	
#if defined(DEBUG) && !defined(AUTH_HPW) 
	fprintf(NetTrace, "\r\nasked_key=%d, sent_key=%d, pke=%s,\r\n",
		asked_key, sent_key,
		pke_status((char **)0) ? "on" : "off");
#endif
	return doit;
}


int
pke_do()
{
	int st = pke_status(NULL);

	if ((st & PKE_OK) == 0) {
		send_wont(TELOPT_PKE, 0);
		return 0;
	}
	send_key(st);
	
	return 1;
}

/*
 * We received IS from server.
 * We generate a private key, and encrypt it with server's public key
 * (data) and return it to server.  The hard bit is all in
 * privkey_gen(). 
 */
int
pke_is(data, cnt)
	unsigned char *data;
	int cnt;
{
#ifdef PKE_GENRSA
	unsigned char buf[512];
	unsigned char *key;
	int klen, len, k2 = 0, old = 1;

	data[cnt] = '\0';			/* null terminate the key */

	switch (data[0]) {
	case PKE_SR1:
		break;
	case PKE_SR2:
		break;
	case PKE_2KEY:
		k2 = 1;				/* FALLTHROUGH */
	case PKE_KEY:
		data++;				/* FALLTHROUGH */
		old = 0;
	default:
		key = pke_privgen(data, k2);
		if ((klen = strlen(key)) < 500) {
			if (old)		/* old server */
				sprintf(buf, "%c%c%c%c%s%c%c",
					IAC, SB, TELOPT_PKE, TELQUAL_IS,
					key, IAC, SE);
			else
				sprintf(buf, "%c%c%c%c%c%s%c%c",
					IAC, SB, TELOPT_PKE, TELQUAL_IS,
					(k2) ? PKE_2KEY : PKE_KEY,
					key, IAC, SE);
				
			len = strlen((char *)buf+4) + 4; /* buf[3] is 0 ... */
			writenet(buf, len);
			printsub('>', &buf[2], len - 2);
			sent_key = 1;
		}
		break;
	}
#endif
	return 0;
}
#endif
