/* NAME:
 *	fwtkauth.c - challenge/response api using the TIS fwtk auth server
 *
 * DESCRIPTION:
 *	These routines are used in place of null_challenge() and
 *	null_verify(). They simply interface to the TIS auth server.
 *
 * SEE ALSO:
 *	hpw.c
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This code is presented AS IS.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char rcsid[] = "from Id: doauth.c,v 1.1 1995/02/08 11:50:16 sjg Exp ";
static char RCSid[] = "$Id: fwtkauth.c,v 1.2 1997/02/08 07:13:27 sjg Exp $";
#endif
#ifndef NO_FWTK

#include <stdio.h>
#include <syslog.h>

#include "firewall.h"				/* TIS fwtk */

#undef auth_send

static int AuthSrvOk = 0;

char *
fwtk_challenge(user, prog, err)
	char *user, *prog, **err;
{
	static Cfg *confp = NULL;
	static char buf[256];
	
	AuthSrvOk = 0;
	
	if (confp == NULL) {
		if ((confp = cfg_read(prog)) == (Cfg *)-1)
			exit(1);
	}
	auth_open(confp);

	/* get welcome message from auth server */
	if (auth_recv(buf,sizeof(buf))) {
		auth_close();
		return NULL;
	}
	if(strncmp(buf,"Authsrv ready",13)) {
		auth_close();
		if (err)
			*err = buf;
		return NULL;
	}

	AuthSrvOk = 1;
	
	(void) sprintf(buf, "authorize %s %s", user, prog);
	auth_send(buf);
	auth_recv(buf,sizeof(buf));
	/* see what we got */
	if (strncmp(buf,"challenge ",10) == 0)
		return &buf[10];
	else if (strncmp(buf,"password",8) == 0)
		return buf;
	/* anything else is likely an error */
	if (err)
		*err = buf;
	return NULL;
}

int
fwtk_verify(user, response, err)
	char *user, *response, **err;
{
	static char buf[256];
	int sts = 0;

	if (err)
		*err = NULL;

	if (AuthSrvOk) {
		(void) sprintf(buf,"response '%s'", response);
		auth_send(buf);
		auth_recv(buf,sizeof(buf));
		auth_send("exit");		/* say good bye */
		auth_close();

		if (strncmp(buf,"ok",2) == 0)
			sts = 1;		/* OK */
		else if (err)
			*err = buf;
	}
	return sts;
}

#ifdef MAIN
int
main(argc, argv)
	int argc;
	char **argv;
{
	char buf[128];
	char *cp, *err;
	int i;
	
	
	if (cp = fwtk_challenge(getenv("USER"), "stelnetd", &err)) {
		fprintf(stderr, "%s", cp);
		if (fgets(buf, sizeof (buf), stdin))
			cp = (fwtk_verify("", buf, &err) > 0) ? "Ok" : err;
	} else
		cp = err;
	fprintf(stderr, "%s\n", cp);
	exit(0);
}
#endif
#endif
