/* NAME:
 *	telnet_getpw - get user password
 *
 * DESCRIPTION:
 *	This interface can be replaced with whatever is needed to get
 *	a password for a user.  If the password database is
 *	/etc/passwd then we just use the system routines.
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1995, Simon J. Gerraty.
 *      
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: getpw.c,v 1.2 1997/02/08 07:13:28 sjg Exp $";
#endif

#include <sys/param.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <unistd.h>
#include <time.h>
#if defined(sun) && defined(__svr4__)
#include <shadow.h>
# define SOLARIS
#endif

#ifndef _PATH_PASSWD
# define _PATH_PASSWD "/etc/passwd"
#endif
char *_path_authpasswd = _PATH_PASSWD;

static struct passwd *
getstpw(user)
	char *user;
{
	FILE *fp;
	static struct passwd pw;
	static char pwbuf[256];

	if ((fp = fopen(_path_authpasswd, "r")) == NULL)
		return NULL;
	while (fgets(pwbuf, sizeof (pwbuf) - 1, fp)) {
		if ((pw.pw_name = strtok(pwbuf, ":")) != NULL &&
		    strcmp(user, pw.pw_name) == 0) {

			pw.pw_passwd = strtok(NULL, ":");
			pw.pw_uid = atoi(strtok(NULL, ":"));
			(void) fclose(fp);
			return &pw;
		}
	}
	(void) fclose(fp);
	return NULL;	
}
	
struct passwd *
telnet_getpw(user)
	char *user;
{
#ifdef SOLARIS
	struct spwd *spw;
#endif
	struct passwd *pw;

	/*
	 * handle OS specifics
	 */
	if (strcmp(_path_authpasswd, "/etc/passwd") == 0) {

		pw = getpwnam(user);
#ifdef SOLARIS
		spw = getspnam(user);
		pw->pw_passwd = spw->sp_pwdp;
#endif
		return pw;
	}
	/*
	 * use our own file
	 */
	return getstpw(user);
}

