/* NAME:
 *	hex_dump - print data in hex and ASCII.
 *
 * DESCRIPTION:
 *	The above says it all :-)
 *	
 */
/*
 *      @(#)Copyright (c) 1993, Simon J. Gerraty.
 *      
 *      This code is presented AS IS.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: hexdump.c,v 1.5 1998/06/19 02:50:43 sjg Exp $";
#endif

#include <stdio.h>

int HexOffset = 1;


char *
sprint_hex(dest, buf, nbytes, off)
	char *dest;
	unsigned char *buf;
	int nbytes;
	int off;
{
	register char *rcp = dest;
	register int i;

	if (off && (off % 128) == 0) {
		(void) sprintf(rcp, "\n");
		rcp++;
	}
	if (HexOffset) 
		(void) sprintf(rcp, "0x%04x  ", off);
	else
		(void) sprintf(rcp, "%4d  ", off);
	rcp += strlen(rcp);

	for (i = 0; i < 16 && i < nbytes; i++) {
		(void) sprintf(rcp, "%02x%s", buf[i], (i % 2) ? " " : "");
		rcp += strlen(rcp);
	}
	return dest;
}

char *
shex_dump(dest, buf, nbytes)
	char *dest;
	unsigned char *buf;
	int nbytes;
{
	register unsigned int i, j, off, c;
	unsigned char *endp = buf + nbytes;
	register char *rcp, *cp = dest;
	
	dest[0] = '\0';
	for (i = 0, off = 0; buf < endp; buf += i, off += i) {
		rcp = sprint_hex(cp, buf, endp - buf, off);
		rcp += strlen(rcp);
		while ((rcp - cp) < 56)
			*rcp++ = ' ';
		*rcp++ = '|';

		for (i = 0; i < 16 && buf + i < endp; i++) {
			if ((c = (unsigned int) buf[i]) < ' ' || c >= 0x7f)
				*rcp++ = '.';
			else {
				(void) sprintf(rcp, "%c", c);
				rcp++;
			}
		}
		for (j = i; j < 16; j++)
			*rcp++ = ' ';
		*rcp++ = '|';
		*rcp++ = '\n';
		cp = rcp;
	}
	return dest;
}

int
fprint_hex(fh, buf, nbytes, off)
	FILE *fh;
	register unsigned char *buf;
	register int nbytes;
	int off;
{
	register int i;

	if (off && (off % 128) == 0)
		(void) fprintf(fh, "\n");
	if (HexOffset)
		(void) fprintf(fh, "0x%04x\t", off);
	else
		(void) fprintf(fh, "%4d\t", off);

	for (i = 0; i < 16 && i < nbytes; i++)
		(void) fprintf(fh, "%02x%s", buf[i], (i % 2) ? " " : "");
	return i;
}

void
fhex_dump(fh, buf, nbytes)
	FILE *fh;
	register unsigned char *buf;
	register int nbytes;
{
	register unsigned int i, off, c;
	unsigned char *endp = buf + nbytes;

	if (nbytes > 0)
		(void) fprintf(fh, "\nData: %d bytes\nOffset\tHex\t\t\t\t\t\t ASCII\n",
			      nbytes);
		
	for (i = 0, off = 0; buf < endp; buf += i, off += i) {
		i = fprint_hex(fh, buf, endp - buf, off);
		(void) fprintf(fh, "%*s\t|", (16 - i) * 3, "");
		for (i = 0; i < 16 && buf + i < endp; i++) {
			if ((c = (unsigned int) buf[i]) < ' ' || c >= 0x7f)
				c = '.';
			(void) fprintf(fh, "%c", c);
		}
		(void) fprintf(fh, "%*s|\n", 16 - i, "");
	}
	(void) fprintf(fh, "\n");
	(void) fflush(fh);
}

print_hex(buf, nbytes, off)
	register unsigned char *buf;
	register int nbytes;
	int off;
{
	return fprint_hex(stdout, buf, nbytes, off);
}

void
hex_dump(buf, nbytes)
		register unsigned char *buf;
	register int nbytes;
{
	fhex_dump(stdout, buf, nbytes);
}
	 
#ifdef MAIN
int
main(argc, argv)
	int argc;
	char **argv;
{
	int c;
	char buf[1024+1];
#ifdef TEST
	char buf2[1024+1];
#endif
	
	while ((c = read(0, buf, 1024)) > 0) {
		hex_dump(buf, c);
#ifdef TEST
			shex_dump(buf2, buf, c);
			printf("%s", buf2);
#endif
	}
	return 0;
}
#endif
