/* grossly simplified now that we only use eay's libcrypto */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifdef PKE_GENRSA
#ifndef lint
static char RCSid[] = "$Id: pkepub.c,v 1.3 1997/09/02 12:10:39 sjg Exp $";
#endif

/*
 * telnetd calls us here to generate an RSA pub/private key pair.
 * He sends the public one to client who uses it to encrypt a private
 * session key.
 */
#include <stdio.h>

#include "pkelib.h"
#ifdef USE_SSL
#include "rsa.h"
#include "x509.h"
#include "crypto.h"

static RSA *rsa;
#endif

#ifdef USE_SSL
/* from Eric's genrsa */
static void
cb(p, n)
	int p;
	int n;
{
	int c='*';

	if (p == 0) c='.';
	if (p == 1) c='+';
	if (p >= 2) c='\n';
	fputc(c,stderr);
	fflush(stderr);
#ifdef LINT
	p=n;
#endif
}
#endif

/*
 * Since this key is only used once, a 512 bit key is
 * probably ok.
 */

u_char *
pke_pubgen()
{
	u_char pubkey[2048];
	int ebits = 0;
	int i, j;
#ifdef USE_SSL
	u_char *p = pubkey;
	unsigned long f4=0x10001;		/* significant? */
	
#endif /* USE_SSL */

	CLEAR(pubkey);

	/*
	 * When keybits is 512, we _sometimes_ fail to get
	 * primes!
	 */
	fprintf(stderr, "\r\nGenerating keys (%d bit modulus)\r\n", keybits);
#ifdef USE_SSL
	for (i = 0, rsa = NULL; rsa == NULL && i < 3; i++) {
		rsa = RSA_generate_key(keybits,f4,cb);
	}
	if (rsa == NULL) {
		printf("\r\n");
		fflush(stdout);
		return NULL;
	}
	if ((i = i2d_RSAPublicKey(rsa, &p)) <= 0)
		return NULL;
	return key2str(pubkey, i);	/* ascii string */
#endif /* USE_SSL */
	return NULL;
}

int
pke_privset(privkey, k2)
	u_char *privkey;
	int k2;
{
	u_char key[1024];			/* be safe */
	u_char *priv;
	int i, j;
	
	if (!privkey)
		return -1;

	i = strlen(privkey) / 3;		/* num bits */

	priv = str2key(privkey);

	CLEAR(key);

#ifdef USE_SSL
	if ((j = RSA_private_decrypt(i, priv, key, rsa
#if SSLEAY_VERSION_NUMBER >= 0x0800
				    , RSA_PKCS1_PADDING
#endif
				     )) <= 0)
		return -1;
#endif /* USE_SSL */
	
#ifdef MAIN
	printf("pke_privset(%s), key:", privkey);
	hex_dump(key, i);
#endif
	if (k2)
		de_setkeys(key, &key[sizeof (des_cblock)]); /* separate read/write keys */
	else
		pke_init(key);			/* initialize our engine */
	return 0;
}

#ifdef MAIN

int
tryit(pubkey)
	u_char *pubkey;
{
	u_char *pub, *pkey;

	pub = strdup(pubkey);
	pkey = strdup(pke_privgen(pub));
	
	if (pke_privset(pkey) < 0)
		fprintf(stderr, "pke_privset() failed\n");
	free(pkey);
	free(pub);
}

int
main()
{
	u_char *pubkey;

	if (pubkey = pke_pubgen()) {
		tryit(pubkey);
	}
	else
		fprintf(stderr, "pke_pubgen() failed\n");
	return 0;
}
#endif
#endif /* PKE_GENRSA */
