/* NAME:
 *	x2b - hex to bin, bin to hex
 *
 * SYNOPSIS:
 *	char	*b2x(char *buf, u_char *data, int nbytes);
 *	int	x2b(u_char *buf, char *data, int nbytes);
 *
 * DESCRIPTION:
 *	The function 'b2x' converts "nbytes" of binary data in "data"
 *	to hex string in "buf".  It returns a pointer to "buf".
 *
 *	The function 'x2b' converts up to "nbytes" of hex data in
 *	"data" to binary data in "buf".  It returns the actual number
 *	of bytes loaded into "buf".  If "nbytes" is passed as -1, then
 *	strlen("data") is used to determine "nbytes".
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: x2b.c,v 1.3 1997/08/28 23:41:05 sjg Exp $";
#endif
#include <sys/types.h>
#include <string.h>

char *
b2x(buf, data, nbytes)
	char *buf;
	u_char *data;
	int nbytes;
{
	int i;

	buf[0] = '\0';
	for (i = 0; i < nbytes; i++) {
		(void) sprintf(&buf[i*2], "%02x", data[i]);
	}
	return buf;
}

int
x2b(buf, data, nbytes)
	u_char *buf;
	char *data;
	int nbytes;
{
	int i;
	int c;

	if (nbytes < 0)
		nbytes = strlen(data);
	nbytes /= 2;
	
	for (i = 0; i < nbytes; i++) {
		if (sscanf(&data[i*2], "%02x", &c) < 1)
			break;
		buf[i] = c;
	}
	return i;
}

#ifdef MAIN
int
main(argc, argv)
	int argc;
	char **argv;
{
	char in[1024], out[2050];
	int c, n;
	char *cp;

	if (cp = strrchr(argv[0], '/'))
		cp++;
	else
		cp = argv[0];

	while ((n = read(0, in, sizeof(in))) > 0) {
		if (*cp == 'x') {
			c = x2b(out, in, n);
			write(1, out, c);
		} else {
			printf("%s", b2x(out, in, n));
		}
	}
	exit(0);
}
#endif
