/*
 *	UTMP/WTMP routines.
 */
/*
 *      @(#)Copyright (c) 1995, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: doutmp.c,v 1.9 1998/08/10 02:35:03 sjg Exp $";
#endif

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#ifdef MAIN
# include <stdio.h>
#endif
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <sys/param.h>
#include <syslog.h>

#if !defined(BSD4_4) && !defined(__hpux__)
#include <lastlog.h>
#endif

#if defined(BSD) || defined(ultrix)
# define HAVE_TTYSLOT
#endif
#ifndef L_SET
#define L_SET           SEEK_SET
#endif

#define ARRSTR(a, s) \
	(strncpy((a), s, sizeof (a)))

#if defined(__svr4__) || defined(HAVE_UTMPX_H)
# define UTMPX
# if defined(sun) && !defined(HAVE_GETUTXID)
#   define HAVE_GETUTXID
# endif
#endif

#ifdef UTMPX
# include <utmpx.h>
# define UTMP_T struct utmpx
# define ut_time ut_xtime
# define utmp utmpx
# define HAVE_UT_HOST
# ifdef HAVE_GETUTXID
#   ifndef HAVE_UPDWTMPX
#     define HAVE_UPDWTMPX
#   endif
#   define getutent getutxent
#   define getutid getutxid
#   define getutline getutxline
#   define pututline pututxline
#   define setutent setutxent
#   define endutent endutxent
#   define updwtmp updwtmpx
# endif
# ifdef WTMPX_FILE
#   undef _PATH_WTMP
#   define _PATH_WTMP WTMPX_FILE
# endif
#else
# include <utmp.h>
# define UTMP_T struct utmp
#endif
static UTMP_T ut, *utp = 0;

#ifndef SC_WILDC
# define SC_WILDC 0xff
#endif

do_utmp(utmpf, wtmpf, name, host)
	char *utmpf, *wtmpf, *name, *host;
{
	extern char *ttyname();
	char *line;
	int fd, tty, i;
	int pid = getpid();
	
	memset((char *)&ut, 0, sizeof(ut));
	
	utp = &ut;
	
	line = ttyname(0);
		
	if (line) {
		line += 5;			/* skip /dev/ */
		ARRSTR(ut.ut_line, line);
	}
	
#if defined(HAVE_GETUTID) || defined(HAVE_GETUTXID) || defined(__hpux__) || defined(__svr4__) 
	/*
	 * find entry from parent
	 * we can't use getutid() as we don't know
	 * what it will be, but we do know our pid.
	 */
	ut.ut_type = LOGIN_PROCESS;
	(void) setutent();
	i = 0;
	do {
		utp = getutline(&ut);
#ifdef MAIN
		if (utp) {
			fprintf(stderr, "doutmp[%d] looking at %s,%d,%d\n",
				pid,
				utp->ut_line,
				utp->ut_type, utp->ut_pid);
		}
#endif
#if defined(HAVE_GETUTXID)
		if (utp->ut_pid != pid) {
			utp->ut_type = DEAD_PROCESS;
			modutx(utp);
		}
#endif
		if (i++ > 99) {			/* incase we are invoked wrongly */
			utp = 0;
			break;
		}
#ifdef __hpux__
		if (utp == 0 && ut.ut_line[0] != 'p') {
			/*
			 * <sjg> try looking for pty/line...
			 */
			(void)sprintf(ut.ut_line, "pty/%s", line);
			utp = &ut;
			(void) setutent();
		}
#endif
		
	} while (utp && (utp->ut_pid != pid)) ;

	if (utp) {
		if (name)
			ARRSTR(utp->ut_name, name);
		if (host) {
			ARRSTR(utp->ut_host, host);
#if defined(sun) && defined(__svr4__)
			utp->ut_syslen = strlen(utp->ut_host);
#endif
		}
		utp->ut_type = (name && *name) ? USER_PROCESS : DEAD_PROCESS;
		utp->ut_time = time((long *) 0);
#ifdef __hpux__
		/*
		 * <sjg> while we may have had to prepend pty/ to line so
		 * we could find the utmp entry, we must replace it with
		 * whatever ttyname() returned or logname etc won't work.
		 */
		ARRSTR(utp->ut_line, line);
#endif
#ifdef DEBUG
		syslog(LOG_DEBUG, "pututline(utp), ut_line='%s', ut_name='%s', ut_id='%s', ut_host='%s' ut_pid=%d, ut_type=%d",
		       utp->ut_line, utp->ut_name, utp->ut_id, utp->ut_host,
		       utp->ut_pid, utp->ut_type);
#endif
		/*
		 * save utp content - for wtmp
		 */
		memcpy((char *)&ut, (char *)utp, sizeof(ut));
		
		(void) pututline(utp);
#ifdef DEBUG
		syslog(LOG_DEBUG, "pututline(utp), ut_line='%s', ut_name='%s', ut_id='%s', ut_host='%s' ut_pid=%d, ut_type=%d",
		       utp->ut_line, utp->ut_name, utp->ut_id, utp->ut_host,
		       utp->ut_pid, utp->ut_type);
#endif
#if defined(HAVE_UPDWTMP) || defined(HAVE_UPDWTMPX)
		(void) endutent();
		utp = &ut;
#ifdef DEBUG
		syslog(LOG_DEBUG, "updwtmp(_PATH_WTMP, ut_line='%s', ut_name='%s', ut_id='%s', ut_host='%s' ut_pid=%d, ut_type=%d",
		       utp->ut_line, utp->ut_name, utp->ut_id, utp->ut_host,
		       utp->ut_pid, utp->ut_type);
#endif
		updwtmp(_PATH_WTMP, utp);
		return 0;
#endif
		utmpf = NULL;			/* done */
	} else {
		utp = &ut;
	}
	(void) endutent();
#endif
#if defined(UTMPX) || defined(__hpux__)
	utp->ut_pid = pid;
	if (utp->ut_id[0] == '\0') {
		if ((i = strlen(line)) > sizeof(utp->ut_id)) {
			/*
			 * <sjg> the last few bytes will vary more 
			 * than the first few
			 */
			(void)strncpy(utp->ut_id, &line[i - sizeof(utp->ut_id)],
			      sizeof(utp->ut_id));
			utp->ut_id[0] = 't';	/* telnet */
		} else {
			ARRSTR(utp->ut_id, line);
		}
	}
	utp->ut_type = (name && *name) ? USER_PROCESS : DEAD_PROCESS;
#endif
		
	if (name)
		ARRSTR(utp->ut_name, name);

#if defined(BSD) || defined(sun) || defined(HAVE_UT_HOST)
	if (host) {
		ARRSTR(utp->ut_host, host);
#if defined(sun) && defined(__svr4__)
		utp->ut_syslen = strlen(utp->ut_host);
#endif
	}
#endif
	utp->ut_time = time((long *) 0);
	ARRSTR(utp->ut_line, line);	/* just in case */

	if (utmpf) {
#ifdef DEBUG
		syslog(LOG_DEBUG, "updating %s: %s %s %s", utmpf, line, name, host);
#endif
#ifdef UTMPX
		makeutx(utp);
		return 0;
#else
#ifdef HAVE_TTYSLOT
		tty = ttyslot();
		if (tty > 0 && (fd = open(utmpf, O_WRONLY|O_CREAT, 0644)) >= 0) {
			(void)lseek(fd, (off_t)(tty * sizeof(ut)), L_SET);
#else
		if ((fd = open(utmpf, O_WRONLY|O_APPEND|O_CREAT, 0644)) >= 0) {
#endif
			(void)write(fd, utp, sizeof(ut));
			(void)close(fd);
		}
#endif
	}
#if defined(HAVE_UPDWTMP) || defined(HAVE_UPDWTMPX)
	updwtmp(_PATH_WTMP, utp);
#else
        if (wtmpf && (fd = open(wtmpf, O_WRONLY|O_APPEND, 0)) >= 0) {
#ifdef DEBUG
		syslog(LOG_DEBUG, "updating %s: %s %s %s", wtmpf, line, name, host);
#endif
		(void)write(fd, utp, sizeof(ut));
		(void)close(fd);
	}
#endif
}

/* derrived from NetBSD's login.c */
	
void
do_lastlog(llogf, uid, quiet)
	char *llogf;
	int uid;
	int quiet;
{
#ifndef __hpux__
	struct lastlog ll;
	int fd;

	if (llogf && (fd = open(llogf, O_RDWR, 0)) >= 0) {
		(void)lseek(fd, (off_t)uid * sizeof(ll), L_SET);
		if (!quiet) {
			if (read(fd, (char *)&ll, sizeof(ll)) == sizeof(ll) &&
			    ll.ll_time != 0) {
				(void)printf("Last login: %.*s ",
				    24-5, (char *)ctime(&ll.ll_time));
				if (*ll.ll_host != '\0')
					(void)printf("from %.*s\n",
					    (int)sizeof(ll.ll_host),
					    ll.ll_host);
				else
					(void)printf("on %.*s\n",
					    (int)sizeof(ll.ll_line),
					    ll.ll_line);
			}
			(void)lseek(fd, (off_t)uid * sizeof(ll), L_SET);
		}
		memset((void *)&ll, 0, sizeof(ll));
		(void)time(&ll.ll_time);
		(void)strncpy(ll.ll_line, utp->ut_line, sizeof(ll.ll_line));
#if defined(BSD) || defined(HAVE_UT_HOST)
		if (utp && *utp->ut_host)
			(void)strncpy(ll.ll_host, utp->ut_host, sizeof(ll.ll_host));
#endif
		(void)write(fd, (char *)&ll, sizeof(ll));
		(void)close(fd);
	}
#endif
}
	
#ifdef MAIN
int
main(argc, argv)
	int argc;
	char *argv[];
{
	extern char *ttyname(), *getenv();
#if 0
	char *line;
	int pid = getpid();
#endif
	char *user = getenv("USER");
	char *host = "localhost";

#if 0
	memset((char *)&ut, 0, sizeof(ut));
	
	utp = &ut;
	
	line = ttyname(0);
		
	if (line) {
		line += 5;			/* skip /dev/ */
		ARRSTR(ut.ut_line, line);
	}
#ifdef UTMPX
	ARRSTR(ut.ut_user, ".telnet");
	ut.ut_pid = pid;
	ut.ut_id[0] = 't';
	ut.ut_id[1] = 'n';
	ut.ut_id[2] = SC_WILDC;
	ut.ut_id[3] = SC_WILDC;
	ut.ut_type = LOGIN_PROCESS;
	(void) time(&ut.ut_tv.tv_sec);
	if (makeutx(&ut) == NULL) {
		perror("makeutx");
		exit(1);
	}
#else
	fprintf(stderr, "haven't written this yet!\n");
	exit(1);
#endif
#endif
	if (argc > 1)
		host = argv[1];
	if (argc > 2)
		user = argv[2];
	do_utmp("/tmp/utmp", "/tmp/wtmp", ".telnet", 0);
	do_utmp("/tmp/utmp", "/tmp/wtmp", user, host);
	do_utmp("/tmp/utmp", "/tmp/wtmp", 0, 0);
	exit(0);
}
#endif
