/*
 *	UTMP/WTMP routines.
 */
/*
 *      @(#)Copyright (c) 1995, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: vutmp.c,v 1.5 1998/08/22 08:27:50 sjg Exp $";
#endif

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <sys/param.h>
#include <syslog.h>

#if !defined(BSD4_4) && !defined(__hpux__)
#include <lastlog.h>
#endif

#include "pathnames.h"

#if defined(BSD) || defined(ultrix)
# define HAVE_TTYSLOT
#endif
#ifndef L_SET
#define L_SET           SEEK_SET
#endif

#define ARRSTR(a, s) \
	(strncpy((a), s, sizeof (a)))

#if defined(__svr4__) || defined(HAVE_UTMPX_H)
# define UTMPX
# ifdef sun
# define HAVE_GETUTXID
#endif
#endif

#ifdef UTMPX
# include <utmpx.h>
# define UTMP_T struct utmpx
# ifndef ut_xtime
#   define ut_xtime ut_tv.tv_sec
# endif
# define ut_time ut_xtime
# define utmp utmpx
# define HAVE_UT_HOST
# ifdef HAVE_GETUTXID
# define getutent getutxent
# define getutid getutxid
# define getutline getutxline
# define pututline pututxline
# define setutent setutxent
# define endutent endutxent
# endif
# ifdef WTMPX_FILE
#   undef _PATH_WTMP
#   define _PATH_WTMP WTMPX_FILE
# endif
# ifndef ut_name
#   define ut_name ut_user
# endif
#else
# include <utmp.h>
# define UTMP_T struct utmp
#endif
static UTMP_T ut, *utp = 0;

#ifndef SC_WILDC
# define SC_WILDC 0xff
#endif

#define SHOW_S(f) printf("%s=%.*s\n", #f, sizeof(utp->##f), utp->##f)
#define SHOW_D(f) printf("%s=%d\n", #f, utp->##f)

void
show_ut(utp)
	UTMP_T *utp;
{
	if (utp->ut_name[0] == 0)
		return;
	SHOW_S(ut_name);
	SHOW_S(ut_line);
#if defined(BSD) || defined(sun) || defined(HAVE_UT_HOST)
	SHOW_S(ut_host);
#endif
#ifdef UTMPX
	SHOW_S(ut_id);
	SHOW_D(ut_pid);
	SHOW_D(ut_type);
#endif
	printf("ut_time=%ld\n\n", utp->ut_time);
}
	
int
main(argc, argv)
	int argc;
	char **argv;
{
	FILE *fp;
	char *file = _PATH_UTMP;
	char *line = ttyname(0);

	printf("line=%s\n", line);
#ifdef UTMPX
	if (argc > 1) {
		utmpname(_PATH_WTMP);
	}
	setutent();
	while (utp = getutent()) {
		show_ut(utp);
	}
	endutent();
#else
	if (argc > 1)
		file = argv[1];
	if ((fp = fopen(file, "r")) != 0) {
		while (fread((char *)&ut, sizeof(ut), 1, fp) == 1) {
			show_ut(&ut);
		}
		fclose(fp);
	}
#endif
	exit(0);
}
