/*
 *	Server routines used to negotiate PKE
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifdef PKE_ENCRYPTION
#ifndef lint
static char RCSid[] = "$Id: pkesvr.c,v 1.11 1997/02/08 07:14:05 sjg Exp $";
#endif

#include "telnetd.h"
#include "pkepub.h"

static int got_key = 0;

static u_long A1, A2, B1, B2;

/*
 * We come here first, to tell client what we feel about it all...
 */
int
pke_server(must)
	int must;
{

	pke_off();				/* to be sure */
	got_key = 0;
	
	if (must == 0) {
		set_my_want_state_dont(TELOPT_PKE);
		set_my_want_state_wont(TELOPT_PKE);
		return 0;
	}
	if (must == 1)
		send_will(TELOPT_PKE, 1);
	else
		send_do(TELOPT_PKE, 1);
	return 0;
}


/*
 * We have received SEND from client
 * so we'll generate and send a public key.
 */
int
pke_send(data, cnt)
	unsigned char *data;
	int cnt;
{
#ifdef PKE_GENRSA
	unsigned char buf[512];
	unsigned char *key;
	int klen, k2 = 0, len;

	key = pke_pubgen();

	if (!key) {
		netflush();
		exit(1);
	}
	if (cnt > 0)
		k2 = 1;
	/* we send PKE_2KEY so client knows that we support it */
	if ((klen = strlen(key)) < 500) {
		sprintf(buf, "%c%c%c%c%c%s%c%c",
			IAC, SB, TELOPT_PKE, TELQUAL_IS,
			k2 ? PKE_2KEY : PKE_KEY,
			key, IAC, SE);
		len = strlen((char *)buf+4) + 4;	/* buf[3] is 0 ... */
	
		writenet(buf, len);
	}
#endif
	return 0;
}

/*
 * We have received IS from client.
 * This should contain the private key...
 * We should probaly test that it is secure...
 */
pke_is(data, cnt)
	unsigned char *data;
	int cnt;
{
#ifdef PKE_GENRSA
	unsigned char buf[512];
	char *p;
	int k2 = 0;
	
	data[cnt] = '\0';

	switch (data[0]) {
	case PKE_SR1:
		break;
	case PKE_SR2:
		break;
	case PKE_2KEY:
		k2 = 1;				/* FALLTHROUGH */
	case PKE_KEY:
		data++;				/* FALLTHROUGH */
	default:
		if (pke_privset(data, k2) < 0) {
			/* could not decode key */
			netflush();
			if (must_encrypt > 1)
				exit(1);
			send_dont(TELOPT_PKE, 1);	/* let him know... */
		} else {
			got_key = 1;
		}
		break;
	}
#endif
	return 0;
}

/*
 * this is just a synchronization mechanism
 */
int
pke_will()
{
	int st;

	st = pke_status(NULL);

	got_key = (st & PKE_KEYSET);
	
	if (got_key) {
		got_key++;
	}
	return (must_encrypt > 0);
}

pke_dont_wont()
{
	if (must_encrypt > 1)
		exit(1);
	pke_off();
	return 0;
}

int
pke_check(must)
	int must;
{
	char *p = NULL;
	int st, len;

	st = pke_status(NULL);
	if ((his_state_is_do(TELOPT_PKE) || his_state_is_will(TELOPT_PKE)) &&
	    (st & PKE_KEYSET) != 0) {			/* got key */
		extern int ncc;
		
		/*
		 * This just makes telnetd dag around until the
		 * synching of PKE is complete...
		 */
		netflush();
		/*
		 * This DO tells him we're happy
		 */
		send_do(TELOPT_PKE, 0);		
		got_key = 1;
		while (got_key == 1) {
			ttloop();
		}
		/*
		 * we received a WILL which says he's ready
		 * and so are we...  this last WILL tells him to turn
		 * it on becuase we're about to...
		 */
		send_will(TELOPT_PKE, 0);
		netflush();			/* send it now */
		pke_on();
	}
	st = pke_status(&p);
	if ((st & PKE_CRYPTING) == 0 && must > 1) {
		if (p) {
			len = strlen(p);
			writenet(p, len);
			netflush();
		}	
		exit(1);
	}
	return st;
}

#endif
